package com.nisum.controller;


import com.nisum.dto.ProductDto;
import com.nisum.service.ProductService;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

@RestController

@RequestMapping("/products")
public class ProductController {

    @Autowired
    private ProductService service;

    @GetMapping("/allProducts")
    public Flux<ProductDto> getAllProducts(){
        return service.getAllProducts();
    }

    @GetMapping("/{id}")
    public Mono<ProductDto> getProduct(@PathVariable String id){
        return service.getProduct(id);
    }

    @GetMapping("/price/{id}")
    public Mono<Double> getProductPrice(@PathVariable String id){
        return service.getProductPrice(id);
    }

    @GetMapping("/product-range")
    public Flux<ProductDto> getProductBetweenRange(@RequestParam("min") double min,@RequestParam double max){
        return service.getProductInRange(min, max);
    }


    /*@PostMapping("/addProduct")
    public Mono<ResponseEntity<ProductDto>> saveProduct(@RequestBody Mono<ProductDto> productDtoMono) {
        return productDtoMono
                .flatMap(productDto -> service.saveProduct(Mono.just(productDto)))
                .map(savedProduct -> ResponseEntity.status(HttpStatus.CREATED).body(savedProduct));
    }*/
    @PostMapping("/addProducts")
    public Mono<ResponseEntity<List<ProductDto>>> saveProducts(@RequestBody @Valid Flux<ProductDto> productDtoFlux) {
        return productDtoFlux
                .flatMap(productDto -> service.saveProduct(Mono.just(productDto))) // Save each product
                .collectList() // Collect them into a List
                .map(savedProducts -> ResponseEntity.status(HttpStatus.CREATED).body(savedProducts)); // Return 201 Created
    }


    @PutMapping ("/update/{id}")
    public Mono<ProductDto> updateProduct(@RequestBody Mono<ProductDto> productDtoMono, @PathVariable String id){
        return service.updateProduct(productDtoMono,id);

    }

    @DeleteMapping("/delete/{id}")
    public Mono<Void> deleteProduct(@PathVariable String id){
        return service.deleteProduct(id);
    }



}
