package com.nisum.entity;


import jakarta.validation.constraints.*;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;



@Data
@AllArgsConstructor
@NoArgsConstructor
@Document(collection = "products")
public class Product{

    @Id
    private String id;

    @NotNull(message = "Product name cannot be null")
    @NotBlank(message = "Name cannot be blank or null.")
    @Pattern(regexp = "^[a-zA-Z0-9 ]+$", message = "Name must not contain special characters.")
    private String name;

    @NotNull(message = "Qty should not be null,You should have order at least one ")
    private int qty;

    @NotNull(message = "Price cannot be null")
    @Min(value = 0, message = "Price must be greater than or equal to 0")
    @Max(value = 10000, message = "Price should be less than or equal to 10000")
    private double price;

   /* public Product(String id, String name, int qty) {
        this.id=id;
        this.name=name;
        this.qty=qty;
    }*/


    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getQty() {
        return qty;
    }

    public void setQty(int qty) {
        this.qty = qty;
    }

    public double getPrice() {
        return price;
    }

    public void setPrice(double price) {
        this.price = price;
    }
}
