package com.nisum.service;

import com.nisum.dto.ProductDto;
import com.nisum.exception.DuplicateProductNameException;
import com.nisum.repository.ProductRepository;
import com.nisum.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class ProductService {

    @Autowired
    private ProductRepository repository;


    public Flux<ProductDto> getAllProducts(){
        return repository.findAll().map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> getProduct(String id){
        return repository.findById(id).map(AppUtils::entityToDto);
    }

    public Mono<Double> getProductPrice(String id){
        return repository.findById(id).map(AppUtils::entityToDto).map(dto -> dto.getPrice());
    }

    public Flux<ProductDto> getProductInRange(double min,double max){
        return repository.findByPriceBetween(Range.closed(min,max));
    }

  /*  public Mono<ProductDto> saveProduct(Mono<ProductDto> productDtoMono){
      return  productDtoMono.map(AppUtils::dtoToEntity)
                .flatMap(repository::insert) //flatmap because it is using Mono of mono list
                .map(AppUtils::entityToDto);

    }*/
  public Mono<ProductDto> saveProduct(Mono<ProductDto> productDtoMono) {
      return productDtoMono
              .flatMap(productDto -> repository.findByName(productDto.getName())
                      .flatMap(existingProduct -> Mono.error(new DuplicateProductNameException(
                              "Product with name" + productDto.getName() + "already exists")))) // Throw a custom exception
              .switchIfEmpty(productDtoMono) // Proceed if the product name is unique
              .map(productDto -> AppUtils.dtoToEntity((ProductDto) productDto))
// Convert DTO to Entity
              .flatMap(repository::insert)  // Save the new product entity
              .map(AppUtils::entityToDto);  // Convert entity back to DTO
  }

    public Mono<ProductDto> updateProduct(Mono<ProductDto> productDtoMono,String id){
        return  repository.findById(id)
                .flatMap(p-> productDtoMono.map(AppUtils::dtoToEntity)
                .doOnNext(e -> e.setId(id)))
                .flatMap(repository::save)
                .map(AppUtils::entityToDto);
    }

    public Mono<Void> deleteProduct(String id){
        return repository.deleteById(id);

    }


}
